@@ -410,7 +410,7 @@ BOOL sCommand_add_arg(sCommand* self, MANAGED char* buf, int env, int quoted_str
         int new_size = self->mArgsSize * 2;
     
         if(new_size >= XYZSH_ARG_SIZE_MAX) {
-            err_msg("argument size overflow", sname, sline, self->mArgs[0]);
+            err_msg("argument size overflow", sname, sline);
             return FALSE;
         }
 
@@ -445,7 +445,7 @@ BOOL sCommand_add_block(sCommand* self, sObject* block, char* sname, int sline)
             int new_block_size = self->mBlocksSize * 2;
 
             if(new_block_size >= XYZSH_BLOCK_SIZE_MAX) {
-                err_msg("block size overflow", sname, sline, self->mArgs[0]);
+                err_msg("block size overflow", sname, sline);
                 return FALSE;
             }
             self->mBlocks = REALLOC(self->mBlocks, sizeof(sObject*)*new_block_size);
@@ -476,7 +476,7 @@ BOOL sCommand_add_message(sCommand* self, MANAGED char* message, char* sname, in
             int new_message_size = self->mMessagesSize * 2;
 
             if(new_message_size >= XYZSH_MESSAGE_SIZE_MAX) {
-                err_msg("message size overflow", sname, sline, self->mArgs[0]);
+                err_msg("message size overflow", sname, sline);
                 return FALSE;
             }
 
@@ -505,7 +505,7 @@ BOOL sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* init_val
             self->mEnvsSize *= 2;
 
             if(self->mEnvsSize >= XYZSH_ENV_SIZE_MAX) {
-                err_msg("environment size overflow", sname, sline, self->mArgs[0]);
+                err_msg("environment size overflow", sname, sline);
                 return FALSE;
             }
             self->mEnvs = REALLOC(self->mEnvs, sizeof(sEnv)*self->mEnvsSize);
@@ -533,7 +533,7 @@ BOOL sCommand_add_env_block(sCommand* self, sObject* block, eLineField lf, char*
             self->mEnvsSize *= 2;
 
             if(self->mEnvsSize >= XYZSH_ENV_SIZE_MAX) {
-                err_msg("environment size overflow", sname, sline, self->mArgs[0]);
+                err_msg("environment size overflow", sname, sline);
                 return FALSE;
             }
             self->mEnvs = REALLOC(self->mEnvs, sizeof(sEnv)*self->mEnvsSize);
@@ -563,7 +563,7 @@ BOOL sCommand_add_redirect(sCommand* self, MANAGED char* name, BOOL env, BOOL qu
             const int redirect_size = self->mRedirectsSize;
             
             if(self->mRedirectsSize*2 >= XYZSH_REDIRECT_SIZE_MAX) {
-                err_msg("redirect size overflow", sname, sline, self->mArgs[0]);
+                err_msg("redirect size overflow", sname, sline);
                 return FALSE;
             }
 
@@ -713,17 +713,28 @@ sStatment* sStatment_new(sObject* block, int sline, char* sname)
     SBLOCK(block).mStatmentsNum++;
     self->mLine = sline;
     self->mFName = STRDUP(sname);
+    self->mFlags = 0;
 
     return self;
 }
 
 static void sStatment_copy_deeply(sStatment* dest, sStatment* source)
 {
-    dest->mCommandsNum = source->mCommandsNum;
+    if(source->mFlags & STATMENT_FLAGS_KIND_NODETREE) {
+        dest->mNodeTreeNum = source->mNodeTreeNum;
 
-    int i;
-    for(i=0; i<source->mCommandsNum; i++) {
-        sCommand_copy_deeply(dest->mCommands + i, source->mCommands + i);
+        int i;
+        for(i=0; i<source->mNodeTreeNum; i++) {
+            dest->mNodeTree[i] = ALLOC sNodeTree_clone(source->mNodeTree[i]);
+        }
+    }
+    else {
+        dest->mCommandsNum = source->mCommandsNum;
+
+        int i;
+        for(i=0; i<source->mCommandsNum; i++) {
+            sCommand_copy_deeply(dest->mCommands + i, source->mCommands + i);
+        }
     }
 
     if(source->mFName) {
@@ -739,11 +750,21 @@ static void sStatment_copy_deeply(sStatment* dest, sStatment* source)
 
 static void sStatment_copy_deeply_stack(sStatment* dest, sStatment* source)
 {
-    dest->mCommandsNum = source->mCommandsNum;
+    if(source->mFlags & STATMENT_FLAGS_KIND_NODETREE) {
+        dest->mNodeTreeNum = source->mNodeTreeNum;
 
-    int i;
-    for(i=0; i<source->mCommandsNum; i++) {
-        sCommand_copy_deeply_stack(dest->mCommands + i, source->mCommands + i);
+        int i;
+        for(i=0; i<source->mNodeTreeNum; i++) {
+            dest->mNodeTree[i] = ALLOC sNodeTree_clone(source->mNodeTree[i]);
+        }
+    }
+    else {
+        dest->mCommandsNum = source->mCommandsNum;
+
+        int i;
+        for(i=0; i<source->mCommandsNum; i++) {
+            sCommand_copy_deeply_stack(dest->mCommands + i, source->mCommands + i);
+        }
     }
 
     if(source->mFName) {
@@ -760,15 +781,18 @@ static void sStatment_copy_deeply_stack(sStatment* dest, sStatment* source)
 void sStatment_delete(sStatment* self)
 {
     if(self->mFlags & STATMENT_FLAGS_KIND_NODETREE) {
-        sNodeTree_free(self->mNodeTree);
+        int i;
+        for(i=0; i<self->mNodeTreeNum; i++) {
+            sNodeTree_free(self->mNodeTree[i]);
+        }
     }
     else {
         int i;
         for(i=0; i<self->mCommandsNum; i++) {
             sCommand_delete(self->mCommands + i);
         }
-        if(self->mFName) FREE(self->mFName);
     }
+    if(self->mFName) FREE(self->mFName);
 }
 
 sObject* block_new_on_gc(BOOL user_object)
